#include "bintest.h"

#define ITERATIONS 10000

void write_to_file(char * buf, int size);
ETERM * build_big_erl(int iters);
char * encode_erl(ETERM * erl);


int main(int argc, char ** argv) {
  ETERM * erl = build_big_erl(ITERATIONS);
  char * buf = encode_erl(erl);
  write_to_file(buf,erl_term_len(erl));
  return 1;
}

/* Writes buffer to disk as binary */
void write_to_file(char * buf, int size) {
  FILE * file = fopen("goodbinary","wb");
  
  int n = fwrite(buf,size,1,file);
  
  if(n == 1) {
    printf("Successfully wrote %d bytes to file 'goodbinary'\n",size);
  } else {
    printf("Error writing %d bytes to file\n",size);
  }
  
  fclose(file);
}

/* Builds a large sized ETERM */
ETERM * build_big_erl(int iters) {
  ETERM * biglist[iters];
  ETERM * innerlist[2];
  ETERM * rowtup[2];
  ETERM * datatup[2];
  char guid[50];
  char dataint[2];
  int i;
  
  ETERM * oktuple[3];
  
  ETERM * bigtuple[2];
  
  erl_init(NULL, 0);
  
  /*
    {hbase, {ok, "", [
        [{row,"GUID"},{data,"1"}],...
    ]}}
  
  */
  
  strcpy(guid,"00000000-0000-0000-0000-000000000000");
  sprintf(dataint,"%d",1);
  
  for(i=0;i<iters;i++) {
    /* row tuple */
    rowtup[0] = erl_mk_atom("row");
    rowtup[1] = erl_mk_binary(guid,36);

    /* data tuple */
    datatup[0] = erl_mk_atom("data");
    datatup[1] = erl_mk_binary(dataint,1);

    /* inner list */
    innerlist[0] = erl_mk_tuple(rowtup,2);
    innerlist[1] = erl_mk_tuple(datatup,2);
    biglist[i] = erl_mk_list(innerlist,2);
  }
  
  oktuple[0] = erl_mk_atom("ok");
  oktuple[1] = erl_mk_binary(dataint,0);
  oktuple[2] = erl_mk_list(biglist,iters);
  
  bigtuple[0] = erl_mk_atom("hbase");
  bigtuple[1] = erl_mk_tuple(oktuple,3);
  
  return erl_mk_tuple(bigtuple,2);
}

/* Encodes the specified ETERM into binary */
char * encode_erl(ETERM * erl) {
  int size = erl_term_len(erl);
  char * buf = malloc(size);
  int n = erl_encode(erl,(unsigned char *)buf);
  
  if(n == 1) {
    printf("erl_encode returned an error!\n");
    exit(0);
  }
  
  printf("Encoding erlang to buffer of size %d\n",size);
  
  return buf;
}

